from pydantic import Field

from aiommost.schemas import Schema


class Post(Schema):
    uid: str = Field(alias='id')
    user_id: str
    channel_id: str
    root_id: str
    original_id: str
    message: str



pytest_plugins = (
    'tests.fixtures.client',
    'tests.fixtures.fake',
    'tests.fixtures.channels',
    'tests.fixtures.users',
)


def pytest_make_parametrize_id(config, val):  # noqa: W0613
    return repr(val)



import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.channels.schemas import Channel


class ChannelsClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create_direct(self, first: str, second: str) -> Channel:
        """Create direct channel between users."""
        url = '/channels/direct'
        request = [first, second]

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        channel = orjson.loads(response.content)
        return Channel(**channel)



import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.users.schemas import User


class UsersClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create(self, username: str, email: str, password: str) -> User:
        """Add new user with specified username."""
        url = '/users'
        request = {
            'username': username,
            'email': email,
            'password': password,
        }

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        user = orjson.loads(response.content)
        return User(**user)

    async def delete(self, uid: str, permanent: bool = False) -> None:
        url = f'/users/{uid}'

        response = await self.session.delete(url, params={'permanent': permanent})
        errors.validate(response)

    async def get_by_usernames(self, usernames: list[str]) -> list[User]:
        """Get users with specified usernames."""
        url = '/users/usernames'
        request = usernames

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        users = orjson.loads(response.content)
        return [User(**user) for user in users]

    async def get_by_username(self, username: str) -> User:
        """Get user with specified username."""
        url = f'/users/username/{username}'

        response = await self.session.get(url)
        errors.validate(response)

        user = orjson.loads(response.content)
        return User(**user)



import httpx

from aiommost.channels.client import ChannelsClient
from aiommost.posts.client import PostClient
from aiommost.users.client import UsersClient


class MattermostClient:

    def __init__(self, host: str, token: str, verify: bool | str = True) -> None:
        self.token = token
        self.verify = verify

        self.session = httpx.AsyncClient(
            base_url=f'{host}/api/v4',
            headers={
                'Authorization': f'Bearer {token}',
                'Content-Type': 'application/json',
            },
            verify=verify,
        )

        self.channels = ChannelsClient(self.session)
        self.posts = PostClient(self.session)
        self.users = UsersClient(self.session)

    async def close(self) -> None:
        await self.session.aclose()



import pytest

from aiommost.client import MattermostClient
from aiommost.users.schemas import User


@pytest.fixture()
def create_direct_channel(client: MattermostClient, create_user):
    async def inner(user_id: str = None):
        if not user_id:
            user: User = await create_user()
            user_id = user.uid

        return await client.channels.create_direct(
            first=user_id,
            second=user_id,
        )

    return inner



import orjson
from httpx import Response, codes


class ClientError(Exception):
    """Mattermost client error."""


class BadRequestError(ClientError):
    """Bad request error."""

    def __init__(self, code: str, message: str, detailed: str, request_id: str) -> None:
        super().__init__(f'[{request_id}] {message}')
        self.code = code
        self.request_id = request_id
        self.message = message
        self.detailed = detailed


def validate(response: Response) -> None:
    if response.status_code == codes.BAD_REQUEST:
        error = orjson.loads(response.content)
        raise BadRequestError(
            code=error['id'],
            request_id=error['request_id'],
            message=error['message'],
            detailed=error['detailed_error'],
        )

    response.raise_for_status()



from pydantic import Field

from aiommost.schemas import Schema


class User(Schema):
    uid: str = Field(alias='id')
    username: str
    email: str
    firstname: str = Field(alias='first_name')
    lastname: str = Field(alias='last_name')
    nickname: str



import pytest

from aiommost.client import MattermostClient


@pytest.fixture()
def create_user(client: MattermostClient, fake):
    async def inner(username: str = None, email: str = None, password: str = None):
        return await client.users.create(
            username=username or fake.pystr().lower(),
            email=email or fake.email(),
            password=password or fake.password(),
        )

    return inner



import pytest

from aiommost.client import MattermostClient
from aiommost.users.schemas import User
from aiommost.errors import BadRequestError


async def test_user_created(client: MattermostClient, fake):
    username, email, password = fake.pystr().lower(), fake.email(), fake.password()

    user = await client.users.create(username, email, password)
    assert user.username == username


async def test_user_deleted(client: MattermostClient, fake):
    username, email, password = fake.pystr(), fake.email(), fake.password()

    user = await client.users.create(username, email, password)

    await client.users.delete(user.uid)


async def test_get_by_username_found(client: MattermostClient, create_user):
    existing_user: User = await create_user()

    user = await client.users.get_by_username(existing_user.username)
    assert user.username == existing_user.username


async def test_get_by_username_not_found(fake, client: MattermostClient):
    with pytest.raises(BadRequestError):
        await client.users.get_by_username(fake.pystr())


async def test_get_by_usernames(client: MattermostClient, create_user):
    first_user: User = await create_user()
    second_user: User = await create_user()

    users = await client.users.get_by_usernames([first_user.username, second_user.username])
    assert len(users) == 2



from typing import Any

import orjson
from httpx import AsyncClient

from aiommost import errors
from aiommost.posts.schemas import Post


class PostClient:

    def __init__(self, session: AsyncClient) -> None:
        self.session = session

    async def create(
        self,
        channel_id: str,
        message: str,
        root_id: str = None,
        file_ids: list[str] = None,
        props: dict[str, Any] = None,
    ) -> Post:
        """Create new post in the channel."""
        url = '/posts'
        request = {
            'channel_id': channel_id,
            'message': message,
            'root_id': root_id,
            'file_ids': file_ids,
            'props': props,
        }

        response = await self.session.post(url, content=orjson.dumps(request))
        errors.validate(response)

        post = orjson.loads(response.content)
        return Post(**post)



import os

import pytest

from aiommost.client import MattermostClient


@pytest.fixture
def client():
    host = os.environ['TEST_MM_HOST']
    token = os.environ['TEST_MM_TOKEN']

    return MattermostClient(host, token)



from aiommost.client import MattermostClient
from aiommost.channels.schemas import Channel


async def test_post_created(fake, client: MattermostClient, create_direct_channel):
    channel: Channel = await create_direct_channel()
    await client.posts.create(
        channel_id=channel.uid,
        message=fake.pystr(),
    )



from datetime import datetime
from typing import Optional

from pydantic import Field

from aiommost.schemas import Schema


class Channel(Schema):
    uid: str = Field(alias='id')
    team_id: str
    category: str = Field(alias='type')
    name: str
    display_name: str
    header: str
    purpose: str
    msg_count: Optional[int] = Field(alias='total_msg_count')
    creator_id: str
    last_post_at: datetime



from aiommost.client import MattermostClient
from aiommost import schemas


async def test_direct_channel_created(client: MattermostClient, create_user):
    user: schemas.User = await create_user()
    await client.channels.create_direct(user.uid, user.uid)
